/**
 * @file main.c
 * @author Jake G
 * @date 15 June 2024
 * @brief File contains the main function.
 *
 * This file contains the main logic loop and exists to setup the values 
 * specific to the micro-controller. All other functions and configuration are
 * extracted into separate source files and headers for configuration.
 */

#define F_CPU 3333333UL

//This can prevent issues with utils/delay.h library with the gcc toolchain
#define __DELAY_BACKWARD_COMPATIBLE__

#include "config.h"
#include "RegEdit.h"
#include "zero_cross_detection.h"
#include "ADC.h"
#include "TriacOut.h"
#include "load.h"
#include "Enable.h"
#include <avr/cpufunc.h> /* Required header file */
#include <avr/io.h>
#include <util/delay.h>



//Set the function pointer for the delay func
void (*Delay_MicroSeconds)(double us) = _delay_us;

static void setup_adc0(void)
{
    //Set the bits used.
    ADC0.CTRLA = 0x0;
    
    //Set the sample accumulation number to 32
    ADC0.CTRLB = 0x5;
    
    //Set the Voltage Reference point to VCC.
    ADC0.CTRLC |= (1<<4);
    
    //Set the Prescaler to divide the F_PER by 32
    ADC0.CTRLC |= (1<<2);
    
    //Setup a initialization delay
    ADC0.CTRLD |= (2<<5);
    
    //Add ADC variation for cleaner readings
    ADC0.CTRLD |= (1<<4);
    
    //Enable the ADC
    ADC0.CTRLA |= (1<<0);
}

static void setup_pa4_adc(void)
{
    //Set the PA4 pin as an input pin.
    PORTA.DIR ^= ~(1<<4);
    
    //Configure the MUXPOS or which pin gets connected to ADC0 periph.
    ADC0.MUXPOS = 0x04; //AIN4 
}

static uint16_t read_adc0(void)
{
    ADC0.COMMAND = ADC_STCONV_bm;
    
    //Wait until the sampling/conversion is done.      
    while (!(ADC0.INTFLAGS & ADC_RESRDY_bm))
    {
        ;
    }
    
    //Bit Shift to account for the accumulated samples.
    //AKA we divide by 32
    uint16_t value = ADC0.RES >> 5;
    return value;
}


int main(int argc, char **argv)
{ 

    Enable_SetPinsLow();
    TriacOut_SetupPins();
    ADC_Setup();

    while(true){
        for(int i = 0; i < GatePulsesQty; i++){
            ZCD_Poll();
            _delay_us(Tau);
            TriacOut_SetAllHigh();  //Only G1 exists in High power mode
            TriacOut_PulsePins(GatePulses[i]);
        }
        
        //The G1 pin is low at this point.
        _delay_ms(2500);
        ZCD_Poll();

        //Enable pins are enabled(set high) if the ADCLOAD value is valid.
        Load_HandleLoadPortA(ADC_LOAD1, EN1);
        Load_HandleLoadPortB(ADC_LOAD2, EN2);
        Load_HandleLoadPortB(ADC_LOAD3, EN3);
        
        while(true){
            //;   //Do nothing until new Power cycle/reset occurs
        }
    }
}
